# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: yamcs/protobuf/packets/packets_service.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from yamcs.api import annotations_pb2 as yamcs_dot_api_dot_annotations__pb2
from yamcs.api import httpbody_pb2 as yamcs_dot_api_dot_httpbody__pb2
from yamcs.protobuf.packets import packets_pb2 as yamcs_dot_protobuf_dot_packets_dot_packets__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='yamcs/protobuf/packets/packets_service.proto',
  package='yamcs.protobuf.packets',
  syntax='proto2',
  serialized_options=_b('\n\022org.yamcs.protobufB\023PacketsServiceProtoP\001'),
  serialized_pb=_b('\n,yamcs/protobuf/packets/packets_service.proto\x12\x16yamcs.protobuf.packets\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1byamcs/api/annotations.proto\x1a\x18yamcs/api/httpbody.proto\x1a$yamcs/protobuf/packets/packets.proto\"*\n\x16ListPacketNamesRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\"\'\n\x17ListPacketNamesResponse\x12\x0c\n\x04name\x18\x01 \x03(\t\"\xc2\x01\n\x12ListPacketsRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x0b\n\x03pos\x18\x02 \x01(\x03\x12\r\n\x05limit\x18\x03 \x01(\x05\x12\r\n\x05order\x18\x04 \x01(\t\x12\x0c\n\x04name\x18\x05 \x03(\t\x12\x0c\n\x04next\x18\x06 \x01(\t\x12)\n\x05start\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12(\n\x04stop\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"f\n\x13ListPacketsResponse\x12\x34\n\x06packet\x18\x01 \x03(\x0b\x32$.yamcs.protobuf.packets.TmPacketData\x12\x19\n\x11\x63ontinuationToken\x18\x02 \x01(\t\"a\n\x10GetPacketRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12+\n\x07gentime\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0e\n\x06seqnum\x18\x03 \x01(\x05\"\x8b\x01\n\x14StreamPacketsRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12)\n\x05start\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12(\n\x04stop\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04name\x18\x04 \x03(\t\"d\n\x13\x45xportPacketRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12+\n\x07gentime\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0e\n\x06seqnum\x18\x03 \x01(\x05\"\x8b\x01\n\x14\x45xportPacketsRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12)\n\x05start\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12(\n\x04stop\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04name\x18\x04 \x03(\t\"N\n\x17SubscribePacketsRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x0e\n\x06stream\x18\x02 \x01(\t\x12\x11\n\tprocessor\x18\x03 \x01(\t\"P\n\x1aSubscribeContainersRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x11\n\tprocessor\x18\x02 \x01(\t\x12\r\n\x05names\x18\x03 \x03(\t\"\x94\x01\n\rContainerData\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x32\n\x0egenerationTime\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x31\n\rreceptionTime\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0e\n\x06\x62inary\x18\x04 \x01(\x0c\x32\x8f\t\n\nPacketsApi\x12\x9e\x01\n\x0fListPacketNames\x12..yamcs.protobuf.packets.ListPacketNamesRequest\x1a/.yamcs.protobuf.packets.ListPacketNamesResponse\"*\x8a\x92\x03&\n$/api/archive/{instance}/packet-names\x12\x8d\x01\n\x0bListPackets\x12*.yamcs.protobuf.packets.ListPacketsRequest\x1a+.yamcs.protobuf.packets.ListPacketsResponse\"%\x8a\x92\x03!\n\x1f/api/archive/{instance}/packets\x12\x95\x01\n\tGetPacket\x12(.yamcs.protobuf.packets.GetPacketRequest\x1a$.yamcs.protobuf.packets.TmPacketData\"8\x8a\x92\x03\x34\n2/api/archive/{instance}/packets/{gentime}/{seqnum}\x12\x9c\x01\n\rStreamPackets\x12,.yamcs.protobuf.packets.StreamPacketsRequest\x1a$.yamcs.protobuf.packets.TmPacketData\"5\x8a\x92\x03\x31\x1a,/api/stream-archive/{instance}:streamPackets:\x01*0\x01\x12\x91\x01\n\x0c\x45xportPacket\x12+.yamcs.protobuf.packets.ExportPacketRequest\x1a\x13.yamcs.api.HttpBody\"?\x8a\x92\x03;\n9/api/archive/{instance}/packets/{gentime}/{seqnum}:export\x12\x81\x01\n\rExportPackets\x12,.yamcs.protobuf.packets.ExportPacketsRequest\x1a\x13.yamcs.api.HttpBody\"+\x8a\x92\x03\'\n%/api/archive/{instance}:exportPackets0\x01\x12z\n\x10SubscribePackets\x12/.yamcs.protobuf.packets.SubscribePacketsRequest\x1a$.yamcs.protobuf.packets.TmPacketData\"\r\xda\x92\x03\t\n\x07packets0\x01\x12\x84\x01\n\x13SubscribeContainers\x12\x32.yamcs.protobuf.packets.SubscribeContainersRequest\x1a%.yamcs.protobuf.packets.ContainerData\"\x10\xda\x92\x03\x0c\n\ncontainers0\x01\x42+\n\x12org.yamcs.protobufB\x13PacketsServiceProtoP\x01')
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,yamcs_dot_api_dot_annotations__pb2.DESCRIPTOR,yamcs_dot_api_dot_httpbody__pb2.DESCRIPTOR,yamcs_dot_protobuf_dot_packets_dot_packets__pb2.DESCRIPTOR,])




_LISTPACKETNAMESREQUEST = _descriptor.Descriptor(
  name='ListPacketNamesRequest',
  full_name='yamcs.protobuf.packets.ListPacketNamesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.packets.ListPacketNamesRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=198,
  serialized_end=240,
)


_LISTPACKETNAMESRESPONSE = _descriptor.Descriptor(
  name='ListPacketNamesResponse',
  full_name='yamcs.protobuf.packets.ListPacketNamesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='yamcs.protobuf.packets.ListPacketNamesResponse.name', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=242,
  serialized_end=281,
)


_LISTPACKETSREQUEST = _descriptor.Descriptor(
  name='ListPacketsRequest',
  full_name='yamcs.protobuf.packets.ListPacketsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.packets.ListPacketsRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pos', full_name='yamcs.protobuf.packets.ListPacketsRequest.pos', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='limit', full_name='yamcs.protobuf.packets.ListPacketsRequest.limit', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='order', full_name='yamcs.protobuf.packets.ListPacketsRequest.order', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='yamcs.protobuf.packets.ListPacketsRequest.name', index=4,
      number=5, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='next', full_name='yamcs.protobuf.packets.ListPacketsRequest.next', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start', full_name='yamcs.protobuf.packets.ListPacketsRequest.start', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stop', full_name='yamcs.protobuf.packets.ListPacketsRequest.stop', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=284,
  serialized_end=478,
)


_LISTPACKETSRESPONSE = _descriptor.Descriptor(
  name='ListPacketsResponse',
  full_name='yamcs.protobuf.packets.ListPacketsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='packet', full_name='yamcs.protobuf.packets.ListPacketsResponse.packet', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='continuationToken', full_name='yamcs.protobuf.packets.ListPacketsResponse.continuationToken', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=480,
  serialized_end=582,
)


_GETPACKETREQUEST = _descriptor.Descriptor(
  name='GetPacketRequest',
  full_name='yamcs.protobuf.packets.GetPacketRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.packets.GetPacketRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gentime', full_name='yamcs.protobuf.packets.GetPacketRequest.gentime', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seqnum', full_name='yamcs.protobuf.packets.GetPacketRequest.seqnum', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=584,
  serialized_end=681,
)


_STREAMPACKETSREQUEST = _descriptor.Descriptor(
  name='StreamPacketsRequest',
  full_name='yamcs.protobuf.packets.StreamPacketsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.packets.StreamPacketsRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start', full_name='yamcs.protobuf.packets.StreamPacketsRequest.start', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stop', full_name='yamcs.protobuf.packets.StreamPacketsRequest.stop', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='yamcs.protobuf.packets.StreamPacketsRequest.name', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=684,
  serialized_end=823,
)


_EXPORTPACKETREQUEST = _descriptor.Descriptor(
  name='ExportPacketRequest',
  full_name='yamcs.protobuf.packets.ExportPacketRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.packets.ExportPacketRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='gentime', full_name='yamcs.protobuf.packets.ExportPacketRequest.gentime', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seqnum', full_name='yamcs.protobuf.packets.ExportPacketRequest.seqnum', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=825,
  serialized_end=925,
)


_EXPORTPACKETSREQUEST = _descriptor.Descriptor(
  name='ExportPacketsRequest',
  full_name='yamcs.protobuf.packets.ExportPacketsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.packets.ExportPacketsRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start', full_name='yamcs.protobuf.packets.ExportPacketsRequest.start', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stop', full_name='yamcs.protobuf.packets.ExportPacketsRequest.stop', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='yamcs.protobuf.packets.ExportPacketsRequest.name', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=928,
  serialized_end=1067,
)


_SUBSCRIBEPACKETSREQUEST = _descriptor.Descriptor(
  name='SubscribePacketsRequest',
  full_name='yamcs.protobuf.packets.SubscribePacketsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.packets.SubscribePacketsRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stream', full_name='yamcs.protobuf.packets.SubscribePacketsRequest.stream', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='processor', full_name='yamcs.protobuf.packets.SubscribePacketsRequest.processor', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1069,
  serialized_end=1147,
)


_SUBSCRIBECONTAINERSREQUEST = _descriptor.Descriptor(
  name='SubscribeContainersRequest',
  full_name='yamcs.protobuf.packets.SubscribeContainersRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.packets.SubscribeContainersRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='processor', full_name='yamcs.protobuf.packets.SubscribeContainersRequest.processor', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='names', full_name='yamcs.protobuf.packets.SubscribeContainersRequest.names', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1149,
  serialized_end=1229,
)


_CONTAINERDATA = _descriptor.Descriptor(
  name='ContainerData',
  full_name='yamcs.protobuf.packets.ContainerData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='yamcs.protobuf.packets.ContainerData.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='generationTime', full_name='yamcs.protobuf.packets.ContainerData.generationTime', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='receptionTime', full_name='yamcs.protobuf.packets.ContainerData.receptionTime', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='binary', full_name='yamcs.protobuf.packets.ContainerData.binary', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1232,
  serialized_end=1380,
)

_LISTPACKETSREQUEST.fields_by_name['start'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_LISTPACKETSREQUEST.fields_by_name['stop'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_LISTPACKETSRESPONSE.fields_by_name['packet'].message_type = yamcs_dot_protobuf_dot_packets_dot_packets__pb2._TMPACKETDATA
_GETPACKETREQUEST.fields_by_name['gentime'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_STREAMPACKETSREQUEST.fields_by_name['start'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_STREAMPACKETSREQUEST.fields_by_name['stop'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_EXPORTPACKETREQUEST.fields_by_name['gentime'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_EXPORTPACKETSREQUEST.fields_by_name['start'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_EXPORTPACKETSREQUEST.fields_by_name['stop'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_CONTAINERDATA.fields_by_name['generationTime'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_CONTAINERDATA.fields_by_name['receptionTime'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
DESCRIPTOR.message_types_by_name['ListPacketNamesRequest'] = _LISTPACKETNAMESREQUEST
DESCRIPTOR.message_types_by_name['ListPacketNamesResponse'] = _LISTPACKETNAMESRESPONSE
DESCRIPTOR.message_types_by_name['ListPacketsRequest'] = _LISTPACKETSREQUEST
DESCRIPTOR.message_types_by_name['ListPacketsResponse'] = _LISTPACKETSRESPONSE
DESCRIPTOR.message_types_by_name['GetPacketRequest'] = _GETPACKETREQUEST
DESCRIPTOR.message_types_by_name['StreamPacketsRequest'] = _STREAMPACKETSREQUEST
DESCRIPTOR.message_types_by_name['ExportPacketRequest'] = _EXPORTPACKETREQUEST
DESCRIPTOR.message_types_by_name['ExportPacketsRequest'] = _EXPORTPACKETSREQUEST
DESCRIPTOR.message_types_by_name['SubscribePacketsRequest'] = _SUBSCRIBEPACKETSREQUEST
DESCRIPTOR.message_types_by_name['SubscribeContainersRequest'] = _SUBSCRIBECONTAINERSREQUEST
DESCRIPTOR.message_types_by_name['ContainerData'] = _CONTAINERDATA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ListPacketNamesRequest = _reflection.GeneratedProtocolMessageType('ListPacketNamesRequest', (_message.Message,), dict(
  DESCRIPTOR = _LISTPACKETNAMESREQUEST,
  __module__ = 'yamcs.protobuf.packets.packets_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.packets.ListPacketNamesRequest)
  ))
_sym_db.RegisterMessage(ListPacketNamesRequest)

ListPacketNamesResponse = _reflection.GeneratedProtocolMessageType('ListPacketNamesResponse', (_message.Message,), dict(
  DESCRIPTOR = _LISTPACKETNAMESRESPONSE,
  __module__ = 'yamcs.protobuf.packets.packets_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.packets.ListPacketNamesResponse)
  ))
_sym_db.RegisterMessage(ListPacketNamesResponse)

ListPacketsRequest = _reflection.GeneratedProtocolMessageType('ListPacketsRequest', (_message.Message,), dict(
  DESCRIPTOR = _LISTPACKETSREQUEST,
  __module__ = 'yamcs.protobuf.packets.packets_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.packets.ListPacketsRequest)
  ))
_sym_db.RegisterMessage(ListPacketsRequest)

ListPacketsResponse = _reflection.GeneratedProtocolMessageType('ListPacketsResponse', (_message.Message,), dict(
  DESCRIPTOR = _LISTPACKETSRESPONSE,
  __module__ = 'yamcs.protobuf.packets.packets_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.packets.ListPacketsResponse)
  ))
_sym_db.RegisterMessage(ListPacketsResponse)

GetPacketRequest = _reflection.GeneratedProtocolMessageType('GetPacketRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETPACKETREQUEST,
  __module__ = 'yamcs.protobuf.packets.packets_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.packets.GetPacketRequest)
  ))
_sym_db.RegisterMessage(GetPacketRequest)

StreamPacketsRequest = _reflection.GeneratedProtocolMessageType('StreamPacketsRequest', (_message.Message,), dict(
  DESCRIPTOR = _STREAMPACKETSREQUEST,
  __module__ = 'yamcs.protobuf.packets.packets_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.packets.StreamPacketsRequest)
  ))
_sym_db.RegisterMessage(StreamPacketsRequest)

ExportPacketRequest = _reflection.GeneratedProtocolMessageType('ExportPacketRequest', (_message.Message,), dict(
  DESCRIPTOR = _EXPORTPACKETREQUEST,
  __module__ = 'yamcs.protobuf.packets.packets_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.packets.ExportPacketRequest)
  ))
_sym_db.RegisterMessage(ExportPacketRequest)

ExportPacketsRequest = _reflection.GeneratedProtocolMessageType('ExportPacketsRequest', (_message.Message,), dict(
  DESCRIPTOR = _EXPORTPACKETSREQUEST,
  __module__ = 'yamcs.protobuf.packets.packets_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.packets.ExportPacketsRequest)
  ))
_sym_db.RegisterMessage(ExportPacketsRequest)

SubscribePacketsRequest = _reflection.GeneratedProtocolMessageType('SubscribePacketsRequest', (_message.Message,), dict(
  DESCRIPTOR = _SUBSCRIBEPACKETSREQUEST,
  __module__ = 'yamcs.protobuf.packets.packets_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.packets.SubscribePacketsRequest)
  ))
_sym_db.RegisterMessage(SubscribePacketsRequest)

SubscribeContainersRequest = _reflection.GeneratedProtocolMessageType('SubscribeContainersRequest', (_message.Message,), dict(
  DESCRIPTOR = _SUBSCRIBECONTAINERSREQUEST,
  __module__ = 'yamcs.protobuf.packets.packets_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.packets.SubscribeContainersRequest)
  ))
_sym_db.RegisterMessage(SubscribeContainersRequest)

ContainerData = _reflection.GeneratedProtocolMessageType('ContainerData', (_message.Message,), dict(
  DESCRIPTOR = _CONTAINERDATA,
  __module__ = 'yamcs.protobuf.packets.packets_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.packets.ContainerData)
  ))
_sym_db.RegisterMessage(ContainerData)


DESCRIPTOR._options = None

_PACKETSAPI = _descriptor.ServiceDescriptor(
  name='PacketsApi',
  full_name='yamcs.protobuf.packets.PacketsApi',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=1383,
  serialized_end=2550,
  methods=[
  _descriptor.MethodDescriptor(
    name='ListPacketNames',
    full_name='yamcs.protobuf.packets.PacketsApi.ListPacketNames',
    index=0,
    containing_service=None,
    input_type=_LISTPACKETNAMESREQUEST,
    output_type=_LISTPACKETNAMESRESPONSE,
    serialized_options=_b('\212\222\003&\n$/api/archive/{instance}/packet-names'),
  ),
  _descriptor.MethodDescriptor(
    name='ListPackets',
    full_name='yamcs.protobuf.packets.PacketsApi.ListPackets',
    index=1,
    containing_service=None,
    input_type=_LISTPACKETSREQUEST,
    output_type=_LISTPACKETSRESPONSE,
    serialized_options=_b('\212\222\003!\n\037/api/archive/{instance}/packets'),
  ),
  _descriptor.MethodDescriptor(
    name='GetPacket',
    full_name='yamcs.protobuf.packets.PacketsApi.GetPacket',
    index=2,
    containing_service=None,
    input_type=_GETPACKETREQUEST,
    output_type=yamcs_dot_protobuf_dot_packets_dot_packets__pb2._TMPACKETDATA,
    serialized_options=_b('\212\222\0034\n2/api/archive/{instance}/packets/{gentime}/{seqnum}'),
  ),
  _descriptor.MethodDescriptor(
    name='StreamPackets',
    full_name='yamcs.protobuf.packets.PacketsApi.StreamPackets',
    index=3,
    containing_service=None,
    input_type=_STREAMPACKETSREQUEST,
    output_type=yamcs_dot_protobuf_dot_packets_dot_packets__pb2._TMPACKETDATA,
    serialized_options=_b('\212\222\0031\032,/api/stream-archive/{instance}:streamPackets:\001*'),
  ),
  _descriptor.MethodDescriptor(
    name='ExportPacket',
    full_name='yamcs.protobuf.packets.PacketsApi.ExportPacket',
    index=4,
    containing_service=None,
    input_type=_EXPORTPACKETREQUEST,
    output_type=yamcs_dot_api_dot_httpbody__pb2._HTTPBODY,
    serialized_options=_b('\212\222\003;\n9/api/archive/{instance}/packets/{gentime}/{seqnum}:export'),
  ),
  _descriptor.MethodDescriptor(
    name='ExportPackets',
    full_name='yamcs.protobuf.packets.PacketsApi.ExportPackets',
    index=5,
    containing_service=None,
    input_type=_EXPORTPACKETSREQUEST,
    output_type=yamcs_dot_api_dot_httpbody__pb2._HTTPBODY,
    serialized_options=_b('\212\222\003\'\n%/api/archive/{instance}:exportPackets'),
  ),
  _descriptor.MethodDescriptor(
    name='SubscribePackets',
    full_name='yamcs.protobuf.packets.PacketsApi.SubscribePackets',
    index=6,
    containing_service=None,
    input_type=_SUBSCRIBEPACKETSREQUEST,
    output_type=yamcs_dot_protobuf_dot_packets_dot_packets__pb2._TMPACKETDATA,
    serialized_options=_b('\332\222\003\t\n\007packets'),
  ),
  _descriptor.MethodDescriptor(
    name='SubscribeContainers',
    full_name='yamcs.protobuf.packets.PacketsApi.SubscribeContainers',
    index=7,
    containing_service=None,
    input_type=_SUBSCRIBECONTAINERSREQUEST,
    output_type=_CONTAINERDATA,
    serialized_options=_b('\332\222\003\014\n\ncontainers'),
  ),
])
_sym_db.RegisterServiceDescriptor(_PACKETSAPI)

DESCRIPTOR.services_by_name['PacketsApi'] = _PACKETSAPI

# @@protoc_insertion_point(module_scope)



from yamcs.core import pagination
from yamcs.core.helpers import to_isostring
from yamcs.protobuf.timeline import timeline_pb2
from yamcs.timeline.model import Band, Item, View


class TimelineClient:
    """
    Client for working with Yamcs timeline.
    """

    def __init__(self, ctx, instance):
        super(TimelineClient, self).__init__()
        self.ctx = ctx
        self._instance = instance

    def list_views(self):
        """
        List the views.

        :rtype: ~collections.abc.Iterable[.View]
        """
        # Server does not do pagination on listings of this resource.
        # Return an iterator anyway for similarity with other API methods
        response = self.ctx.get_proto(f"/timeline/{self._instance}/views")
        message = timeline_pb2.ListViewsResponse()
        message.ParseFromString(response.content)
        return [View(proto) for proto in getattr(message, "views")]

    def get_view(self, id):
        """
        Fetch a view by its identifier.

        :param str id: View identifier
        :rtype: .View
        """
        url = f"/timeline/{self._instance}/views/{id}"
        response = self.ctx.get_proto(url)
        message = timeline_pb2.TimelineView()
        message.ParseFromString(response.content)
        return View(message)

    def save_view(self, view):
        """
        Save or update a view.

        :param .View view: View object
        """
        if view.id:
            url = f"/timeline/{self._instance}/views/{view.id}"
            req = timeline_pb2.UpdateViewRequest()
        else:
            url = f"/timeline/{self._instance}/views"
            req = timeline_pb2.AddViewRequest()

        req.name = view._proto.name
        req.description = view._proto.description
        req.bands[:] = [band.id for band in view._proto.bands]

        if view.id:
            response = self.ctx.put_proto(url, data=req.SerializeToString())
        else:
            response = self.ctx.post_proto(url, data=req.SerializeToString())

        message = timeline_pb2.TimelineView()
        message.ParseFromString(response.content)
        view._proto = message

    def delete_view(self, view):
        """
        Delete a view.

        :param string view: View identifier.
        """
        url = f"/timeline/{self._instance}/views/{view}"
        self.ctx.delete_proto(url)

    def list_bands(self):
        """
        List the bands.

        :rtype: ~collections.abc.Iterable[.Band]
        """
        # Server does not do pagination on listings of this resource.
        # Return an iterator anyway for similarity with other API methods
        response = self.ctx.get_proto(f"/timeline/{self._instance}/bands")
        message = timeline_pb2.ListBandsResponse()
        message.ParseFromString(response.content)
        return [Band._as_subclass(proto) for proto in getattr(message, "bands")]

    def get_band(self, id):
        """
        Fetch a band by its identifier.

        :param str id: Band identifier
        :rtype: .Band
        """
        url = f"/timeline/{self._instance}/bands/{id}"
        response = self.ctx.get_proto(url)
        message = timeline_pb2.TimelineBand()
        message.ParseFromString(response.content)
        return Band._as_subclass(message)

    def save_band(self, band):
        """
        Save or update a band.

        :param .Band band: Band object
        """
        if band.id:
            url = f"/timeline/{self._instance}/bands/{band.id}"
            req = timeline_pb2.UpdateBandRequest()
        else:
            url = f"/timeline/{self._instance}/bands"
            req = timeline_pb2.AddBandRequest()
            req.source = "rdb"
            req.type = band._proto.type

        req.shared = True
        req.name = band._proto.name
        req.description = band._proto.description
        for k in band._proto.tags:
            req.tags.append(k)
        for k, v in band._proto.properties.items():
            req.properties[k] = v

        if band.id:
            response = self.ctx.put_proto(url, data=req.SerializeToString())
        else:
            response = self.ctx.post_proto(url, data=req.SerializeToString())

        message = timeline_pb2.TimelineBand()
        message.ParseFromString(response.content)
        band._proto = message

    def delete_band(self, band):
        """
        Delete a band.

        :param string band: Band identifier.
        """
        url = f"/timeline/{self._instance}/bands/{band}"
        self.ctx.delete_proto(url)

    def list_items(self, band=None, start=None, stop=None, page_size=500):
        """
        List the items.

        :param str band: Return only items matching the specified band
        :param ~datetime.datetime start: Minimum stop time of the returned
                                         items (exclusive)
        :param ~datetime.datetime stop: Maximum start time of the returned
                                        items (exclusive)
        :param int page_size: Page size of underlying requests. Higher values imply
                              less overhead, but risk hitting the maximum message size
                              limit.
        :rtype: ~collections.abc.Iterable[.Item]
        """
        params = {}
        if band is not None:
            params["band"] = band
        if page_size is not None:
            params["limit"] = page_size
        if start is not None:
            params["start"] = to_isostring(start)
        if stop is not None:
            params["stop"] = to_isostring(stop)

        return pagination.Iterator(
            ctx=self.ctx,
            path=f"/timeline/{self._instance}/items",
            params=params,
            response_class=timeline_pb2.ListItemsResponse,
            items_key="items",
            item_mapper=Item,
        )

    def get_item(self, id):
        """
        Fetch an item by its identifier.

        :param str id: Item identifier
        :rtype: .Item
        """
        url = f"/timeline/{self._instance}/items/{id}"
        response = self.ctx.get_proto(url)
        message = timeline_pb2.TimelineItem()
        message.ParseFromString(response.content)
        return Item(message)

    def save_item(self, item):
        """
        Save or update an item.

        :param .Item item: Item object
        """
        if item.id:
            url = f"/timeline/{self._instance}/items/{item.id}"
            req = timeline_pb2.UpdateItemRequest()
        else:
            url = f"/timeline/{self._instance}/items"
            req = timeline_pb2.CreateItemRequest()
            req.type = item._proto.type

        req.name = item._proto.name
        req.tags.MergeFrom(item._proto.tags)
        req.start.MergeFrom(item._proto.start)
        req.duration.MergeFrom(item._proto.duration)

        if item.id:
            response = self.ctx.put_proto(url, data=req.SerializeToString())
        else:
            response = self.ctx.post_proto(url, data=req.SerializeToString())

        message = timeline_pb2.TimelineItem()
        message.ParseFromString(response.content)
        item._proto = message

    def delete_item(self, item):
        """
        Delete an item.

        :param string item: Item identifier.
        """
        url = f"/timeline/{self._instance}/items/{item}"
        self.ctx.delete_proto(url)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: yamcs/protobuf/alarms/alarms_service.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from yamcs.api import annotations_pb2 as yamcs_dot_api_dot_annotations__pb2
from yamcs.protobuf.alarms import alarms_pb2 as yamcs_dot_protobuf_dot_alarms_dot_alarms__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='yamcs/protobuf/alarms/alarms_service.proto',
  package='yamcs.protobuf.alarms',
  syntax='proto2',
  serialized_options=_b('\n\031org.yamcs.protobuf.alarmsB\022AlarmsServiceProtoP\001'),
  serialized_pb=_b('\n*yamcs/protobuf/alarms/alarms_service.proto\x12\x15yamcs.protobuf.alarms\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1byamcs/api/annotations.proto\x1a\"yamcs/protobuf/alarms/alarms.proto\"\xb3\x01\n\x11ListAlarmsRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x0b\n\x03pos\x18\x02 \x01(\x03\x12\r\n\x05limit\x18\x03 \x01(\x05\x12)\n\x05start\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12(\n\x04stop\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\r\n\x05order\x18\x06 \x01(\t\x12\x0c\n\x04name\x18\x07 \x01(\t\"F\n\x12ListAlarmsResponse\x12\x30\n\x06\x61larms\x18\x01 \x03(\x0b\x32 .yamcs.protobuf.alarms.AlarmData\"A\n\x1aListProcessorAlarmsRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x11\n\tprocessor\x18\x02 \x01(\t\"O\n\x1bListProcessorAlarmsResponse\x12\x30\n\x06\x61larms\x18\x01 \x03(\x0b\x32 .yamcs.protobuf.alarms.AlarmData\"=\n\x16SubscribeAlarmsRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x11\n\tprocessor\x18\x02 \x01(\t\"\x8d\x01\n\x10\x45\x64itAlarmRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x11\n\tprocessor\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x0e\n\x06seqnum\x18\x04 \x01(\r\x12\r\n\x05state\x18\x05 \x01(\t\x12\x0f\n\x07\x63omment\x18\x06 \x01(\t\x12\x16\n\x0eshelveDuration\x18\x07 \x01(\x04\"n\n\x17\x41\x63knowledgeAlarmRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x11\n\tprocessor\x18\x02 \x01(\t\x12\r\n\x05\x61larm\x18\x03 \x01(\t\x12\x0e\n\x06seqnum\x18\x04 \x01(\r\x12\x0f\n\x07\x63omment\x18\x05 \x01(\t\"\x81\x01\n\x12ShelveAlarmRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x11\n\tprocessor\x18\x02 \x01(\t\x12\r\n\x05\x61larm\x18\x03 \x01(\t\x12\x0e\n\x06seqnum\x18\x04 \x01(\r\x12\x0f\n\x07\x63omment\x18\x05 \x01(\t\x12\x16\n\x0eshelveDuration\x18\x06 \x01(\x04\"Z\n\x14UnshelveAlarmRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x11\n\tprocessor\x18\x02 \x01(\t\x12\r\n\x05\x61larm\x18\x03 \x01(\t\x12\x0e\n\x06seqnum\x18\x04 \x01(\r\"h\n\x11\x43learAlarmRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x11\n\tprocessor\x18\x02 \x01(\t\x12\r\n\x05\x61larm\x18\x03 \x01(\t\x12\x0e\n\x06seqnum\x18\x04 \x01(\r\x12\x0f\n\x07\x63omment\x18\x05 \x01(\t\"C\n\x1cSubscribeGlobalStatusRequest\x12\x10\n\x08instance\x18\x01 \x01(\t\x12\x11\n\tprocessor\x18\x02 \x01(\t\"\xb2\x01\n\x11GlobalAlarmStatus\x12\x1b\n\x13unacknowledgedCount\x18\x01 \x01(\x05\x12\x1c\n\x14unacknowledgedActive\x18\x02 \x01(\x08\x12\x19\n\x11\x61\x63knowledgedCount\x18\x03 \x01(\x05\x12\x1a\n\x12\x61\x63knowledgedActive\x18\x04 \x01(\x08\x12\x14\n\x0cshelvedCount\x18\x05 \x01(\x05\x12\x15\n\rshelvedActive\x18\x06 \x01(\x08\x32\x95\x0b\n\tAlarmsApi\x12\x90\x01\n\nListAlarms\x12(.yamcs.protobuf.alarms.ListAlarmsRequest\x1a).yamcs.protobuf.alarms.ListAlarmsResponse\"-\x8a\x92\x03)\n\'/api/archive/{instance}/alarms/{name**}\x12\xb1\x01\n\x13ListProcessorAlarms\x12\x31.yamcs.protobuf.alarms.ListProcessorAlarmsRequest\x1a\x32.yamcs.protobuf.alarms.ListProcessorAlarmsResponse\"3\x8a\x92\x03/\n-/api/processors/{instance}/{processor}/alarms\x12\x97\x01\n\tEditAlarm\x12\'.yamcs.protobuf.alarms.EditAlarmRequest\x1a\x16.google.protobuf.Empty\"I\x8a\x92\x03\x45*>/api/processors/{instance}/{processor}/alarms/{name*}/{seqnum}0\x01:\x01*\x12\xb0\x01\n\x10\x41\x63knowledgeAlarm\x12..yamcs.protobuf.alarms.AcknowledgeAlarmRequest\x1a\x16.google.protobuf.Empty\"T\x8a\x92\x03P\x1aK/api/processors/{instance}/{processor}/alarms/{alarm*}/{seqnum}:acknowledge:\x01*\x12\xa1\x01\n\x0bShelveAlarm\x12).yamcs.protobuf.alarms.ShelveAlarmRequest\x1a\x16.google.protobuf.Empty\"O\x8a\x92\x03K\x1a\x46/api/processors/{instance}/{processor}/alarms/{alarm*}/{seqnum}:shelve:\x01*\x12\xa4\x01\n\rUnshelveAlarm\x12+.yamcs.protobuf.alarms.UnshelveAlarmRequest\x1a\x16.google.protobuf.Empty\"N\x8a\x92\x03J\x1aH/api/processors/{instance}/{processor}/alarms/{alarm*}/{seqnum}:unshelve\x12\x9e\x01\n\nClearAlarm\x12(.yamcs.protobuf.alarms.ClearAlarmRequest\x1a\x16.google.protobuf.Empty\"N\x8a\x92\x03J\x1a\x45/api/processors/{instance}/{processor}/alarms/{alarm*}/{seqnum}:clear:\x01*\x12\x93\x01\n\x15SubscribeGlobalStatus\x12\x33.yamcs.protobuf.alarms.SubscribeGlobalStatusRequest\x1a(.yamcs.protobuf.alarms.GlobalAlarmStatus\"\x19\xda\x92\x03\x15\n\x13global-alarm-status0\x01\x12r\n\x0fSubscribeAlarms\x12-.yamcs.protobuf.alarms.SubscribeAlarmsRequest\x1a .yamcs.protobuf.alarms.AlarmData\"\x0c\xda\x92\x03\x08\n\x06\x61larms0\x01\x42\x31\n\x19org.yamcs.protobuf.alarmsB\x12\x41larmsServiceProtoP\x01')
  ,
  dependencies=[google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,yamcs_dot_api_dot_annotations__pb2.DESCRIPTOR,yamcs_dot_protobuf_dot_alarms_dot_alarms__pb2.DESCRIPTOR,])




_LISTALARMSREQUEST = _descriptor.Descriptor(
  name='ListAlarmsRequest',
  full_name='yamcs.protobuf.alarms.ListAlarmsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.alarms.ListAlarmsRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pos', full_name='yamcs.protobuf.alarms.ListAlarmsRequest.pos', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='limit', full_name='yamcs.protobuf.alarms.ListAlarmsRequest.limit', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start', full_name='yamcs.protobuf.alarms.ListAlarmsRequest.start', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stop', full_name='yamcs.protobuf.alarms.ListAlarmsRequest.stop', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='order', full_name='yamcs.protobuf.alarms.ListAlarmsRequest.order', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='yamcs.protobuf.alarms.ListAlarmsRequest.name', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=197,
  serialized_end=376,
)


_LISTALARMSRESPONSE = _descriptor.Descriptor(
  name='ListAlarmsResponse',
  full_name='yamcs.protobuf.alarms.ListAlarmsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='alarms', full_name='yamcs.protobuf.alarms.ListAlarmsResponse.alarms', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=378,
  serialized_end=448,
)


_LISTPROCESSORALARMSREQUEST = _descriptor.Descriptor(
  name='ListProcessorAlarmsRequest',
  full_name='yamcs.protobuf.alarms.ListProcessorAlarmsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.alarms.ListProcessorAlarmsRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='processor', full_name='yamcs.protobuf.alarms.ListProcessorAlarmsRequest.processor', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=450,
  serialized_end=515,
)


_LISTPROCESSORALARMSRESPONSE = _descriptor.Descriptor(
  name='ListProcessorAlarmsResponse',
  full_name='yamcs.protobuf.alarms.ListProcessorAlarmsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='alarms', full_name='yamcs.protobuf.alarms.ListProcessorAlarmsResponse.alarms', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=517,
  serialized_end=596,
)


_SUBSCRIBEALARMSREQUEST = _descriptor.Descriptor(
  name='SubscribeAlarmsRequest',
  full_name='yamcs.protobuf.alarms.SubscribeAlarmsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.alarms.SubscribeAlarmsRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='processor', full_name='yamcs.protobuf.alarms.SubscribeAlarmsRequest.processor', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=598,
  serialized_end=659,
)


_EDITALARMREQUEST = _descriptor.Descriptor(
  name='EditAlarmRequest',
  full_name='yamcs.protobuf.alarms.EditAlarmRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.alarms.EditAlarmRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='processor', full_name='yamcs.protobuf.alarms.EditAlarmRequest.processor', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='yamcs.protobuf.alarms.EditAlarmRequest.name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seqnum', full_name='yamcs.protobuf.alarms.EditAlarmRequest.seqnum', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='state', full_name='yamcs.protobuf.alarms.EditAlarmRequest.state', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='comment', full_name='yamcs.protobuf.alarms.EditAlarmRequest.comment', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='shelveDuration', full_name='yamcs.protobuf.alarms.EditAlarmRequest.shelveDuration', index=6,
      number=7, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=662,
  serialized_end=803,
)


_ACKNOWLEDGEALARMREQUEST = _descriptor.Descriptor(
  name='AcknowledgeAlarmRequest',
  full_name='yamcs.protobuf.alarms.AcknowledgeAlarmRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.alarms.AcknowledgeAlarmRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='processor', full_name='yamcs.protobuf.alarms.AcknowledgeAlarmRequest.processor', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='alarm', full_name='yamcs.protobuf.alarms.AcknowledgeAlarmRequest.alarm', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seqnum', full_name='yamcs.protobuf.alarms.AcknowledgeAlarmRequest.seqnum', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='comment', full_name='yamcs.protobuf.alarms.AcknowledgeAlarmRequest.comment', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=805,
  serialized_end=915,
)


_SHELVEALARMREQUEST = _descriptor.Descriptor(
  name='ShelveAlarmRequest',
  full_name='yamcs.protobuf.alarms.ShelveAlarmRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.alarms.ShelveAlarmRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='processor', full_name='yamcs.protobuf.alarms.ShelveAlarmRequest.processor', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='alarm', full_name='yamcs.protobuf.alarms.ShelveAlarmRequest.alarm', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seqnum', full_name='yamcs.protobuf.alarms.ShelveAlarmRequest.seqnum', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='comment', full_name='yamcs.protobuf.alarms.ShelveAlarmRequest.comment', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='shelveDuration', full_name='yamcs.protobuf.alarms.ShelveAlarmRequest.shelveDuration', index=5,
      number=6, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=918,
  serialized_end=1047,
)


_UNSHELVEALARMREQUEST = _descriptor.Descriptor(
  name='UnshelveAlarmRequest',
  full_name='yamcs.protobuf.alarms.UnshelveAlarmRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.alarms.UnshelveAlarmRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='processor', full_name='yamcs.protobuf.alarms.UnshelveAlarmRequest.processor', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='alarm', full_name='yamcs.protobuf.alarms.UnshelveAlarmRequest.alarm', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seqnum', full_name='yamcs.protobuf.alarms.UnshelveAlarmRequest.seqnum', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1049,
  serialized_end=1139,
)


_CLEARALARMREQUEST = _descriptor.Descriptor(
  name='ClearAlarmRequest',
  full_name='yamcs.protobuf.alarms.ClearAlarmRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.alarms.ClearAlarmRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='processor', full_name='yamcs.protobuf.alarms.ClearAlarmRequest.processor', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='alarm', full_name='yamcs.protobuf.alarms.ClearAlarmRequest.alarm', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seqnum', full_name='yamcs.protobuf.alarms.ClearAlarmRequest.seqnum', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='comment', full_name='yamcs.protobuf.alarms.ClearAlarmRequest.comment', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1141,
  serialized_end=1245,
)


_SUBSCRIBEGLOBALSTATUSREQUEST = _descriptor.Descriptor(
  name='SubscribeGlobalStatusRequest',
  full_name='yamcs.protobuf.alarms.SubscribeGlobalStatusRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance', full_name='yamcs.protobuf.alarms.SubscribeGlobalStatusRequest.instance', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='processor', full_name='yamcs.protobuf.alarms.SubscribeGlobalStatusRequest.processor', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1247,
  serialized_end=1314,
)


_GLOBALALARMSTATUS = _descriptor.Descriptor(
  name='GlobalAlarmStatus',
  full_name='yamcs.protobuf.alarms.GlobalAlarmStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='unacknowledgedCount', full_name='yamcs.protobuf.alarms.GlobalAlarmStatus.unacknowledgedCount', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unacknowledgedActive', full_name='yamcs.protobuf.alarms.GlobalAlarmStatus.unacknowledgedActive', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='acknowledgedCount', full_name='yamcs.protobuf.alarms.GlobalAlarmStatus.acknowledgedCount', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='acknowledgedActive', full_name='yamcs.protobuf.alarms.GlobalAlarmStatus.acknowledgedActive', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='shelvedCount', full_name='yamcs.protobuf.alarms.GlobalAlarmStatus.shelvedCount', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='shelvedActive', full_name='yamcs.protobuf.alarms.GlobalAlarmStatus.shelvedActive', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1317,
  serialized_end=1495,
)

_LISTALARMSREQUEST.fields_by_name['start'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_LISTALARMSREQUEST.fields_by_name['stop'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_LISTALARMSRESPONSE.fields_by_name['alarms'].message_type = yamcs_dot_protobuf_dot_alarms_dot_alarms__pb2._ALARMDATA
_LISTPROCESSORALARMSRESPONSE.fields_by_name['alarms'].message_type = yamcs_dot_protobuf_dot_alarms_dot_alarms__pb2._ALARMDATA
DESCRIPTOR.message_types_by_name['ListAlarmsRequest'] = _LISTALARMSREQUEST
DESCRIPTOR.message_types_by_name['ListAlarmsResponse'] = _LISTALARMSRESPONSE
DESCRIPTOR.message_types_by_name['ListProcessorAlarmsRequest'] = _LISTPROCESSORALARMSREQUEST
DESCRIPTOR.message_types_by_name['ListProcessorAlarmsResponse'] = _LISTPROCESSORALARMSRESPONSE
DESCRIPTOR.message_types_by_name['SubscribeAlarmsRequest'] = _SUBSCRIBEALARMSREQUEST
DESCRIPTOR.message_types_by_name['EditAlarmRequest'] = _EDITALARMREQUEST
DESCRIPTOR.message_types_by_name['AcknowledgeAlarmRequest'] = _ACKNOWLEDGEALARMREQUEST
DESCRIPTOR.message_types_by_name['ShelveAlarmRequest'] = _SHELVEALARMREQUEST
DESCRIPTOR.message_types_by_name['UnshelveAlarmRequest'] = _UNSHELVEALARMREQUEST
DESCRIPTOR.message_types_by_name['ClearAlarmRequest'] = _CLEARALARMREQUEST
DESCRIPTOR.message_types_by_name['SubscribeGlobalStatusRequest'] = _SUBSCRIBEGLOBALSTATUSREQUEST
DESCRIPTOR.message_types_by_name['GlobalAlarmStatus'] = _GLOBALALARMSTATUS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ListAlarmsRequest = _reflection.GeneratedProtocolMessageType('ListAlarmsRequest', (_message.Message,), dict(
  DESCRIPTOR = _LISTALARMSREQUEST,
  __module__ = 'yamcs.protobuf.alarms.alarms_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.alarms.ListAlarmsRequest)
  ))
_sym_db.RegisterMessage(ListAlarmsRequest)

ListAlarmsResponse = _reflection.GeneratedProtocolMessageType('ListAlarmsResponse', (_message.Message,), dict(
  DESCRIPTOR = _LISTALARMSRESPONSE,
  __module__ = 'yamcs.protobuf.alarms.alarms_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.alarms.ListAlarmsResponse)
  ))
_sym_db.RegisterMessage(ListAlarmsResponse)

ListProcessorAlarmsRequest = _reflection.GeneratedProtocolMessageType('ListProcessorAlarmsRequest', (_message.Message,), dict(
  DESCRIPTOR = _LISTPROCESSORALARMSREQUEST,
  __module__ = 'yamcs.protobuf.alarms.alarms_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.alarms.ListProcessorAlarmsRequest)
  ))
_sym_db.RegisterMessage(ListProcessorAlarmsRequest)

ListProcessorAlarmsResponse = _reflection.GeneratedProtocolMessageType('ListProcessorAlarmsResponse', (_message.Message,), dict(
  DESCRIPTOR = _LISTPROCESSORALARMSRESPONSE,
  __module__ = 'yamcs.protobuf.alarms.alarms_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.alarms.ListProcessorAlarmsResponse)
  ))
_sym_db.RegisterMessage(ListProcessorAlarmsResponse)

SubscribeAlarmsRequest = _reflection.GeneratedProtocolMessageType('SubscribeAlarmsRequest', (_message.Message,), dict(
  DESCRIPTOR = _SUBSCRIBEALARMSREQUEST,
  __module__ = 'yamcs.protobuf.alarms.alarms_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.alarms.SubscribeAlarmsRequest)
  ))
_sym_db.RegisterMessage(SubscribeAlarmsRequest)

EditAlarmRequest = _reflection.GeneratedProtocolMessageType('EditAlarmRequest', (_message.Message,), dict(
  DESCRIPTOR = _EDITALARMREQUEST,
  __module__ = 'yamcs.protobuf.alarms.alarms_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.alarms.EditAlarmRequest)
  ))
_sym_db.RegisterMessage(EditAlarmRequest)

AcknowledgeAlarmRequest = _reflection.GeneratedProtocolMessageType('AcknowledgeAlarmRequest', (_message.Message,), dict(
  DESCRIPTOR = _ACKNOWLEDGEALARMREQUEST,
  __module__ = 'yamcs.protobuf.alarms.alarms_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.alarms.AcknowledgeAlarmRequest)
  ))
_sym_db.RegisterMessage(AcknowledgeAlarmRequest)

ShelveAlarmRequest = _reflection.GeneratedProtocolMessageType('ShelveAlarmRequest', (_message.Message,), dict(
  DESCRIPTOR = _SHELVEALARMREQUEST,
  __module__ = 'yamcs.protobuf.alarms.alarms_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.alarms.ShelveAlarmRequest)
  ))
_sym_db.RegisterMessage(ShelveAlarmRequest)

UnshelveAlarmRequest = _reflection.GeneratedProtocolMessageType('UnshelveAlarmRequest', (_message.Message,), dict(
  DESCRIPTOR = _UNSHELVEALARMREQUEST,
  __module__ = 'yamcs.protobuf.alarms.alarms_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.alarms.UnshelveAlarmRequest)
  ))
_sym_db.RegisterMessage(UnshelveAlarmRequest)

ClearAlarmRequest = _reflection.GeneratedProtocolMessageType('ClearAlarmRequest', (_message.Message,), dict(
  DESCRIPTOR = _CLEARALARMREQUEST,
  __module__ = 'yamcs.protobuf.alarms.alarms_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.alarms.ClearAlarmRequest)
  ))
_sym_db.RegisterMessage(ClearAlarmRequest)

SubscribeGlobalStatusRequest = _reflection.GeneratedProtocolMessageType('SubscribeGlobalStatusRequest', (_message.Message,), dict(
  DESCRIPTOR = _SUBSCRIBEGLOBALSTATUSREQUEST,
  __module__ = 'yamcs.protobuf.alarms.alarms_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.alarms.SubscribeGlobalStatusRequest)
  ))
_sym_db.RegisterMessage(SubscribeGlobalStatusRequest)

GlobalAlarmStatus = _reflection.GeneratedProtocolMessageType('GlobalAlarmStatus', (_message.Message,), dict(
  DESCRIPTOR = _GLOBALALARMSTATUS,
  __module__ = 'yamcs.protobuf.alarms.alarms_service_pb2'
  # @@protoc_insertion_point(class_scope:yamcs.protobuf.alarms.GlobalAlarmStatus)
  ))
_sym_db.RegisterMessage(GlobalAlarmStatus)


DESCRIPTOR._options = None

_ALARMSAPI = _descriptor.ServiceDescriptor(
  name='AlarmsApi',
  full_name='yamcs.protobuf.alarms.AlarmsApi',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=1498,
  serialized_end=2927,
  methods=[
  _descriptor.MethodDescriptor(
    name='ListAlarms',
    full_name='yamcs.protobuf.alarms.AlarmsApi.ListAlarms',
    index=0,
    containing_service=None,
    input_type=_LISTALARMSREQUEST,
    output_type=_LISTALARMSRESPONSE,
    serialized_options=_b('\212\222\003)\n\'/api/archive/{instance}/alarms/{name**}'),
  ),
  _descriptor.MethodDescriptor(
    name='ListProcessorAlarms',
    full_name='yamcs.protobuf.alarms.AlarmsApi.ListProcessorAlarms',
    index=1,
    containing_service=None,
    input_type=_LISTPROCESSORALARMSREQUEST,
    output_type=_LISTPROCESSORALARMSRESPONSE,
    serialized_options=_b('\212\222\003/\n-/api/processors/{instance}/{processor}/alarms'),
  ),
  _descriptor.MethodDescriptor(
    name='EditAlarm',
    full_name='yamcs.protobuf.alarms.AlarmsApi.EditAlarm',
    index=2,
    containing_service=None,
    input_type=_EDITALARMREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=_b('\212\222\003E*>/api/processors/{instance}/{processor}/alarms/{name*}/{seqnum}0\001:\001*'),
  ),
  _descriptor.MethodDescriptor(
    name='AcknowledgeAlarm',
    full_name='yamcs.protobuf.alarms.AlarmsApi.AcknowledgeAlarm',
    index=3,
    containing_service=None,
    input_type=_ACKNOWLEDGEALARMREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=_b('\212\222\003P\032K/api/processors/{instance}/{processor}/alarms/{alarm*}/{seqnum}:acknowledge:\001*'),
  ),
  _descriptor.MethodDescriptor(
    name='ShelveAlarm',
    full_name='yamcs.protobuf.alarms.AlarmsApi.ShelveAlarm',
    index=4,
    containing_service=None,
    input_type=_SHELVEALARMREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=_b('\212\222\003K\032F/api/processors/{instance}/{processor}/alarms/{alarm*}/{seqnum}:shelve:\001*'),
  ),
  _descriptor.MethodDescriptor(
    name='UnshelveAlarm',
    full_name='yamcs.protobuf.alarms.AlarmsApi.UnshelveAlarm',
    index=5,
    containing_service=None,
    input_type=_UNSHELVEALARMREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=_b('\212\222\003J\032H/api/processors/{instance}/{processor}/alarms/{alarm*}/{seqnum}:unshelve'),
  ),
  _descriptor.MethodDescriptor(
    name='ClearAlarm',
    full_name='yamcs.protobuf.alarms.AlarmsApi.ClearAlarm',
    index=6,
    containing_service=None,
    input_type=_CLEARALARMREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=_b('\212\222\003J\032E/api/processors/{instance}/{processor}/alarms/{alarm*}/{seqnum}:clear:\001*'),
  ),
  _descriptor.MethodDescriptor(
    name='SubscribeGlobalStatus',
    full_name='yamcs.protobuf.alarms.AlarmsApi.SubscribeGlobalStatus',
    index=7,
    containing_service=None,
    input_type=_SUBSCRIBEGLOBALSTATUSREQUEST,
    output_type=_GLOBALALARMSTATUS,
    serialized_options=_b('\332\222\003\025\n\023global-alarm-status'),
  ),
  _descriptor.MethodDescriptor(
    name='SubscribeAlarms',
    full_name='yamcs.protobuf.alarms.AlarmsApi.SubscribeAlarms',
    index=8,
    containing_service=None,
    input_type=_SUBSCRIBEALARMSREQUEST,
    output_type=yamcs_dot_protobuf_dot_alarms_dot_alarms__pb2._ALARMDATA,
    serialized_options=_b('\332\222\003\010\n\006alarms'),
  ),
])
_sym_db.RegisterServiceDescriptor(_ALARMSAPI)

DESCRIPTOR.services_by_name['AlarmsApi'] = _ALARMSAPI

# @@protoc_insertion_point(module_scope)



import warnings

from yamcs.protobuf.mdb import mdb_pb2


class Algorithm:
    def __init__(self, proto):
        self._proto = proto

    @property
    def name(self):
        """Short name"""
        return self._proto.name

    @property
    def qualified_name(self):
        """Full name (incl. space system)"""
        return self._proto.qualifiedName

    @property
    def aliases(self):
        """List of (namespace, name) pairs, as 2-tuples"""
        return {alias.namespace: alias.name for alias in self._proto.alias}.items()

    @property
    def description(self):
        """Short description."""
        if self._proto.HasField("shortDescription"):
            return self._proto.shortDescription
        return None

    @property
    def long_description(self):
        """Long description."""
        if self._proto.HasField("longDescription"):
            return self._proto.longDescription
        return None

    def __str__(self):
        return self.qualified_name


class Command:
    def __init__(self, proto):
        self._proto = proto

    @property
    def name(self):
        """Short name"""
        return self._proto.name

    @property
    def qualified_name(self):
        """Full name (incl. space system)"""
        return self._proto.qualifiedName

    @property
    def aliases(self):
        """List of (namespace, name) pairs, as 2-tuples"""
        return {alias.namespace: alias.name for alias in self._proto.alias}.items()

    @property
    def description(self):
        """Short description."""
        if self._proto.HasField("shortDescription"):
            return self._proto.shortDescription
        return None

    @property
    def long_description(self):
        """Long description."""
        if self._proto.HasField("longDescription"):
            return self._proto.longDescription
        return None

    @property
    def base_command(self):
        if self._proto.HasField("baseCommand"):
            return Command(self._proto.baseCommand)
        return None

    @property
    def abstract(self):
        """
        Whether this is an abstract command. Abstract commands are
        intended for inheritance and cannot be issued directly.
        """
        return self._proto.abstract

    @property
    def significance(self):
        if self._proto.HasField("significance"):
            return Significance(self._proto.significance)
        return None

    def __str__(self):
        return self.qualified_name


class Significance:
    def __init__(self, proto):
        self._proto = proto

    @property
    def consequence_level(self):
        """
        One of ``NONE``, ``WATCH``, ``WARNING``, ``DISTRESS``, ``CRITICAL``
        or ``SEVERE``.
        """
        if self._proto.HasField("consequenceLevel"):
            return mdb_pb2.SignificanceInfo.SignificanceLevelType.Name(
                self._proto.consequenceLevel
            )
        return None

    @property
    def reason(self):
        """Message attached to this significance."""
        if self._proto.HasField("reasonForWarning"):
            return self._proto.reasonForWarning
        return None

    def __str__(self):
        return f"[{self.consequence_level}] {self.reason}"


class Container:
    def __init__(self, proto):
        self._proto = proto

    @property
    def name(self):
        """Short name"""
        return self._proto.name

    @property
    def qualified_name(self):
        """Full name (incl. space system)"""
        return self._proto.qualifiedName

    @property
    def aliases(self):
        """List of (namespace, name) pairs, as 2-tuples"""
        return {alias.namespace: alias.name for alias in self._proto.alias}.items()

    @property
    def description(self):
        """Short description."""
        if self._proto.HasField("shortDescription"):
            return self._proto.shortDescription
        return None

    @property
    def long_description(self):
        """Long description."""
        if self._proto.HasField("longDescription"):
            return self._proto.longDescription
        return None

    def __str__(self):
        return self.qualified_name


class ArrayType:
    def __init__(self, proto):
        self._proto = proto

    @property
    def name(self):
        """Short name of this type."""
        return self._proto.type.engType

    @property
    def array_type(self):
        """
        In case the elements of an array of this type are also of type `array`, this
        returns type info of the elements' array type.

        .. note::
            This is an uncommon use case. Multi-dimensional arrays are more prevalent.

        :type: :class:`.ArrayType`
        """
        if self._proto.type.HasField("arrayInfo"):
            return ArrayType(self._proto.type.arrayInfo)
        return None

    @property
    def arrayType(self):
        """
        Deprecated. Use ``array_type``.
        """
        warnings.warn(
            "Method renamed for consistency. "
            "Use: array_type() instead of arrayType()",
            FutureWarning,
        )
        return self.array_type

    @property
    def members(self):
        """
        In case the elements of this array are of type `aggregate`, this returns
        an ordered list of its direct sub-members.

        :type: List[:class:`.Member`]
        """
        return [Member(member) for member in self._proto.type.member]

    @property
    def dimensions(self):
        """The number of dimensions in case of a multi-dimensional array."""
        if self._proto.HasField("dimensions"):
            return self._proto.dimensions

    def __str__(self):
        return self.name


class Member:
    """
    A member is a data structure for a specific field of a parent data type
    (either another member, or a parameter of type `aggregate`).

    This is similar to C structs. The top-level of a member hierarchy is
    a parameter of type `aggregate`.
    """

    def __init__(self, proto):
        self._proto = proto

    @property
    def name(self):
        """Short name"""
        return self._proto.name

    @property
    def type(self):
        """
        Engineering type.

        :type: str
        """
        if self._proto.HasField("type"):
            return self._proto.type.engType
        return None

    @property
    def array_type(self):
        """
        In case this member is of type `array`, this returns array-specific
        type info.

        :type: :class:`.ArrayType`
        """
        if self._proto.type.HasField("arrayInfo"):
            return ArrayType(self._proto.type.arrayInfo)
        return None

    @property
    def arrayType(self):
        """
        Deprecated. Use ``array_type``.
        """
        warnings.warn(
            "Method renamed for consistency. "
            "Use: array_type() instead of arrayType()",
            FutureWarning,
        )
        return self.array_type

    @property
    def members(self):
        """
        In case this member is of type `aggregate`, this returns an ordered list
        of its direct sub-members.

        :type: List[:class:`.Member`]
        """
        return [Member(member) for member in self._proto.type.member]

    def __str__(self):
        return self.name


class EnumValue:
    def __init__(self, proto):
        self._proto = proto

    @property
    def value(self):
        """Numeric value"""
        return self._proto.value

    @property
    def label(self):
        """String value"""
        return self._proto.label

    @property
    def description(self):
        """State description"""
        if self._proto.HasField("description"):
            return self.__proto.description
        return None

    def __str__(self):
        return self.label


class DataEncoding:
    def __init__(self, proto):
        self._proto = proto

    @property
    def type(self):
        """Raw type"""
        if self._proto.HasField("type"):
            return mdb_pb2.DataEncodingInfo.Type.Name(self._proto.type)
        return None

    @property
    def little_endian(self):
        """True if little-endian"""
        return self._proto.littleEndian

    @property
    def bitlength(self):
        """The size in bits"""
        if self._proto.HasField("sizeInBits"):
            return self._proto.sizeInBits
        return None

    @property
    def encoding(self):
        """Encoding detail"""
        if self._proto.HasField("encoding"):
            return self._proto.encoding
        return None

    def __str__(self):
        return self.type


class Parameter:
    """
    From XTCE:

        A Parameter is a description of something that can have
        a value. It is not the value itself.
    """

    def __init__(self, proto):
        self._proto = proto

    @property
    def name(self):
        """Short name"""
        return self._proto.name

    @property
    def qualified_name(self):
        """Full name (incl. space system)"""
        return self._proto.qualifiedName

    @property
    def aliases(self):
        """List of (namespace, name) pairs, as 2-tuples"""
        return {alias.namespace: alias.name for alias in self._proto.alias}.items()

    @property
    def description(self):
        """Short description."""
        if self._proto.HasField("shortDescription"):
            return self._proto.shortDescription
        return None

    @property
    def long_description(self):
        """Long description."""
        if self._proto.HasField("longDescription"):
            return self._proto.longDescription
        return None

    @property
    def data_source(self):
        """
        Specifies how this parameter originated (example: ``TELEMETERED``)

        :type: str
        """
        if self._proto.HasField("dataSource"):
            return mdb_pb2.DataSourceType.Name(self._proto.dataSource)
        return None

    @property
    def type(self):
        """
        Engineering type.

        :type: str
        """
        if self._proto.type.HasField("engType"):
            return self._proto.type.engType
        return None

    @property
    def array_type(self):
        """
        In case this parameter is of type `array`, this returns array-specific
        type info.

        :type: :class:`.ArrayType`
        """
        if self._proto.type.HasField("arrayInfo"):
            return ArrayType(self._proto.type.arrayInfo)
        return None

    @property
    def arrayType(self):
        """
        Deprecated. Use ``array_type``.
        """
        warnings.warn(
            "Method renamed for consistency. "
            "Use: array_type() instead of arrayType()",
            FutureWarning,
        )
        return self.array_type

    @property
    def members(self):
        """
        In case this parameter is of type `aggregate`, this returns an ordered list
        of its direct members.

        :type: List[:class:`.Member`]
        """
        return [Member(member) for member in self._proto.type.member]

    @property
    def units(self):
        """
        Engineering unit(s)

        :type: List[str]
        """
        return [info.unit for info in self._proto.type.unitSet]

    @property
    def enum_values(self):
        """
        In case this parameter is of type `enumeration`, this returns an ordered list
        of possible values.

        :type: List[:class:`.EnumValue`]
        """
        return [EnumValue(enumValue) for enumValue in self._proto.type.enumValue]

    @property
    def data_encoding(self):
        """
        Information on the raw encoding of this parameter, if applicable.

        :type: :class:`.DataEncoding`
        """
        if self._proto.type.HasField("dataEncoding"):
            return DataEncoding(self._proto.type.dataEncoding)
        return None

    def __str__(self):
        return self.qualified_name


class SpaceSystem:
    """
    From XTCE:

        A SpaceSystem is a collection of SpaceSystem(s) including space assets,
        ground assets, multi-satellite systems and sub-systems.  A SpaceSystem
        is the root element for the set of data necessary to monitor and command
        an arbitrary space device - this includes the binary decomposition of
        the data streams going into and out of a device.
    """

    def __init__(self, proto):
        self._proto = proto

    @property
    def name(self):
        """Short name"""
        return self._proto.name

    @property
    def qualified_name(self):
        """Full name (incl. space system)"""
        return self._proto.qualifiedName

    @property
    def aliases(self):
        """List of (namespace, name) pairs, as 2-tuples"""
        return {alias.namespace: alias.name for alias in self._proto.alias}.items()

    @property
    def description(self):
        """Short description."""
        if self._proto.HasField("shortDescription"):
            return self._proto.shortDescription
        return None

    @property
    def long_description(self):
        """Long description."""
        if self._proto.HasField("longDescription"):
            return self._proto.longDescription
        return None

    def __str__(self):
        return self.qualified_name



import binascii
import collections.abc
import datetime
import functools
import json
import threading

from yamcs.core.exceptions import YamcsError
from yamcs.core.futures import WebSocketSubscriptionFuture
from yamcs.core.helpers import (
    adapt_name_for_rest,
    to_isostring,
    to_named_object_id,
    to_named_object_ids,
    to_server_time,
)
from yamcs.core.subscriptions import WebSocketSubscriptionManager
from yamcs.protobuf import yamcs_pb2
from yamcs.protobuf.alarms import alarms_pb2, alarms_service_pb2
from yamcs.protobuf.commanding import commanding_pb2, commands_service_pb2
from yamcs.protobuf.mdb import mdb_pb2
from yamcs.protobuf.packets import packets_pb2, packets_service_pb2
from yamcs.protobuf.processing import mdb_override_service_pb2, processing_pb2
from yamcs.protobuf.pvalue import pvalue_pb2
from yamcs.tmtc.model import (
    AlarmUpdate,
    Calibrator,
    CommandHistory,
    ContainerData,
    IssuedCommand,
    MonitoredCommand,
    Packet,
    ParameterData,
    ParameterValue,
    ValueUpdate,
    _parse_alarm,
)


class SequenceGenerator:
    """Static atomic counter."""

    _counter = 0
    _lock = threading.Lock()

    @classmethod
    def next(cls):
        with cls._lock:
            cls._counter += 1
            return cls._counter


def _wrap_callback_parse_parameter_data(subscription, on_data, message):
    """
    Wraps an (optional) user callback to parse ParameterData
    from a WebSocket data message
    """
    pb = processing_pb2.SubscribeParametersData()
    message.Unpack(pb)
    parameter_data = subscription._process(pb)
    if on_data and parameter_data:
        on_data(parameter_data)


def _wrap_callback_parse_packet_data(on_data, message):
    """
    Wraps an (optional) user callback to parse Packet
    from a WebSocket data message
    """
    pb = packets_pb2.TmPacketData()
    message.Unpack(pb)
    packet = Packet(pb)
    on_data(packet)


def _wrap_callback_parse_container_data(subscription, on_data, message):
    """
    Wraps an (optional) user callback to parse ContainerData
    from a WebSocket data message
    """
    pb = packets_service_pb2.ContainerData()
    message.Unpack(pb)
    container_data = subscription._process(pb)
    if on_data:
        on_data(container_data)


def _wrap_callback_parse_cmdhist_data(subscription, on_data, message):
    """
    Wraps an (optional) user callback to parse CommandHistoryEntry
    from a WebSocket data message
    """
    pb = commanding_pb2.CommandHistoryEntry()
    message.Unpack(pb)
    rec = subscription._process(pb)
    if on_data:
        on_data(rec)


def _wrap_callback_parse_alarm_data(subscription, on_data, message):
    """
    Wraps an (optional) user callback to parse Alarm data
    from a WebSocket data message
    """
    pb = alarms_pb2.AlarmData()
    message.Unpack(pb)
    alarm_update = AlarmUpdate(pb)
    subscription._process(alarm_update)
    if on_data:
        on_data(alarm_update)


def _build_value_proto(value):
    proto = yamcs_pb2.Value()
    if isinstance(value, bool):
        proto.type = proto.BOOLEAN
        proto.booleanValue = value
    elif isinstance(value, float):
        proto.type = proto.DOUBLE
        proto.doubleValue = value
    elif isinstance(value, int) and value > 2147483647:
        proto.type = proto.SINT64
        proto.sint64Value = value
    elif isinstance(value, int):
        proto.type = proto.SINT32
        proto.sint32Value = value
    elif isinstance(value, str):
        proto.type = proto.STRING
        proto.stringValue = value
    elif isinstance(value, bytes):
        proto.type = proto.BINARY
        proto.binaryValue = value
    elif isinstance(value, bytearray):
        proto.type = proto.BINARY
        proto.binaryValue = bytes(value)
    elif isinstance(value, datetime.datetime):
        proto.type = proto.TIMESTAMP
        proto.stringValue = to_isostring(value)
    elif isinstance(value, collections.abc.Mapping):
        proto.type = proto.AGGREGATE
        proto.aggregateValue.name.extend(value.keys())
        proto.aggregateValue.value.extend(
            [_build_value_proto(v) for v in value.values()]
        )
    elif isinstance(value, collections.abc.Sequence):
        proto.type = proto.ARRAY
        proto.arrayValue.extend([_build_value_proto(v) for v in value])
    else:
        raise YamcsError("Unrecognized type")
    return proto


def _to_argument_value(value, force_string):
    if isinstance(value, (bytes, bytearray)):
        return binascii.hexlify(value)
    elif isinstance(value, collections.abc.Mapping):
        # Careful to do the JSON dump only at the end,
        # and not at every level of a nested hierarchy
        obj = _compose_aggregate_members(value)
        return json.dumps(obj)
    elif isinstance(value, datetime.datetime):
        return to_isostring(value)
    elif force_string:
        return str(value)
    else:
        return value


def _compose_aggregate_members(value):
    """
    Recursively creates an object that can eventually be serialized to a valid
    aggregate value in JSON. This is a bit different than non-aggregate values,
    because Yamcs is more strict in the values that it accepts (for example:
    unlike regular arguments you cannot assign a numeric string to an integer
    argument, the JSON type needs to be numeric too).
    """
    if isinstance(value, (bytes, bytearray)):
        return binascii.hexlify(value)
    elif isinstance(value, collections.abc.Mapping):
        return {k: _compose_aggregate_members(v) for k, v in value.items()}
    elif isinstance(value, datetime.datetime):
        return to_isostring(value)
    else:
        # No string conversion here, use whatever the user is giving
        return value


def _set_range(ar, range, level):
    ar.level = level
    if range[0]:
        ar.minExclusive = range[0]
    if range[1]:
        ar.maxExclusive = range[1]


def _add_alarms(alarm_info, watch, warning, distress, critical, severe, min_violations):
    alarm_info.minViolations = min_violations

    if watch:
        ar = alarm_info.staticAlarmRange.add()
        _set_range(ar, watch, mdb_pb2.WATCH)
    if warning:
        ar = alarm_info.staticAlarmRange.add()
        _set_range(ar, warning, mdb_pb2.WARNING)
    if distress:
        ar = alarm_info.staticAlarmRange.add()
        _set_range(ar, distress, mdb_pb2.DISTRESS)
    if critical:
        ar = alarm_info.staticAlarmRange.add()
        _set_range(ar, critical, mdb_pb2.CRITICAL)
    if severe:
        ar = alarm_info.staticAlarmRange.add()
        _set_range(ar, severe, mdb_pb2.SEVERE)


def _add_calib(calib_info, type, data):
    type = type.lower()
    if type == Calibrator.POLYNOMIAL:
        calib_info.type = mdb_pb2.CalibratorInfo.POLYNOMIAL
        calib_info.polynomialCalibrator.coefficient.extend(data)
    elif type == Calibrator.SPLINE:
        calib_info.type = mdb_pb2.CalibratorInfo.SPLINE
        spline = mdb_pb2.SplineCalibratorInfo()
        for p in data:
            spi = spline.point.add()
            spi.raw = p[0]
            spi.calibrated = p[1]
    else:
        raise YamcsError("Unrecognized type")


class CommandHistorySubscription(WebSocketSubscriptionFuture):
    """
    Local object providing access to command history updates.

    This object buffers all received command history. This is needed
    to stitch together incremental command history events.

    If you expect to receive a lot of command history updates
    you should periodically clear local cache via ``clear_cache()``.
    In future work, we may add automated buffer management within
    configurable watermarks.

    .. warning::
        If command history updates are received for commands
        that are not currently in the local cache, the returned
        information may be incomplete.
    """

    def __init__(self, manager):
        super(CommandHistorySubscription, self).__init__(manager)
        self._cache = {}

    def clear_cache(self):
        """
        Clears local command history cache.
        """
        self._cache = {}

    def get_command_history(self, issued_command):
        """
        Gets locally cached CommandHistory for the specified command.

        :param .IssuedCommand issued_command: object representing a
                                              previously issued command.
        :rtype: .CommandHistory
        """
        if issued_command.id in self._cache:
            return self._cache[issued_command.id]
        return None

    def _process(self, entry):
        if entry.id in self._cache:
            cmdhist = self._cache[entry.id]
            cmdhist._update(entry.attr)
        else:
            cmdhist = CommandHistory(entry)
            self._cache[entry.id] = cmdhist

        return cmdhist


class ContainerSubscription(WebSocketSubscriptionFuture):
    """
    Local object providing access to container updates
    """

    def __init__(self, manager):
        super(ContainerSubscription, self).__init__(manager)
        self._cache = {}
        """Container cache keyed by container name."""

    def get_container(self, name):
        """
        Returns the latest container of a specific name.

        :param str name: Container name
        :rtype: .ContainerData
        """
        return self._cache[name]

    def list_containers(self):
        """
        Returns the latest container for each name.

        :rtype: .ContainerData[]
        """
        return [self._cache[k] for k in self._cache]

    def _process(self, proto):
        container = ContainerData(proto)
        self._cache[container.name] = container
        return container


class ParameterSubscription(WebSocketSubscriptionFuture):
    """
    Local object representing a subscription of zero or more parameters.

    A subscription object stores the last received value of each
    subscribed parameter.
    """

    def __init__(self, manager):
        super(ParameterSubscription, self).__init__(manager)
        self._mapping = {}
        """Mapping from server-assigned identifier, to requested parameter"""

        self.value_cache = {}
        """Value cache keyed by parameter name."""

        self.delivery_count = 0
        """The number of parameter deliveries."""

    def add(self, parameters, abort_on_invalid=True, send_from_cache=True):
        """
        Add one or more parameters to this subscription.

        :param parameters: Parameter(s) to be added
        :type parameters: Union[str, str[]]
        :param bool abort_on_invalid: If ``True`` one invalid parameter
                                      means any other parameter in the
                                      request will also not be added
                                      to the subscription.
        :param bool send_from_cache: If ``True`` the last processed parameter
                                     value is sent from parameter cache.
                                     When ``False`` only newly processed
                                     parameters are received.
        """
        if not parameters:
            return

        options = processing_pb2.SubscribeParametersRequest()
        options.action = processing_pb2.SubscribeParametersRequest.ADD
        options.abortOnInvalid = abort_on_invalid
        options.sendFromCache = send_from_cache
        options.id.extend(to_named_object_ids(parameters))

        self._manager.send(options)

    def remove(self, parameters):
        """
        Remove one or more parameters from this subscription.

        :param parameters: Parameter(s) to be removed
        :type parameters: Union[str, str[]]
        """
        if not parameters:
            return

        options = processing_pb2.SubscribeParametersRequest()
        options.action = processing_pb2.SubscribeParametersRequest.REMOVE
        options.id.extend(to_named_object_ids(parameters))

        self._manager.send(options)

    def get_value(self, parameter):
        """
        Returns the last value of a specific parameter from local cache.

        :param string parameter: Parameter name.
        :rtype: .ParameterValue
        """
        return self.value_cache[parameter]

    def _process(self, pb):
        # Mapping is only set on the first reply, or whenever
        # the subscription is modifed. Store it in an internal
        # reference, so that we can find it back when receiving
        # future parameter_data updates.
        for k in pb.mapping:
            self._mapping[k] = pb.mapping[k]

        if pb.values:
            self.delivery_count += 1
            parameter_data = ParameterData(pb, self._mapping)
            for pval in parameter_data.parameters:
                self.value_cache[pval.name] = pval
            return parameter_data


class CommandConnection(WebSocketSubscriptionFuture):
    """
    Local object providing access to the acknowledgment progress
    of command updates.

    Only commands issued from this object are monitored.
    """

    def __init__(self, manager, tmtc_client):
        super(CommandConnection, self).__init__(manager)
        self._cmdhist_cache = {}
        self._command_cache = {}
        self._tmtc_client = tmtc_client

    def issue(
        self,
        command,
        args=None,
        dry_run=False,
        comment=None,
        verification=None,
        extra=None,
        beta_args_v2=False,
    ):
        """
        Issue the given command

        :param str command: Either a fully-qualified XTCE name or an alias in the
                            format ``NAMESPACE/NAME``.
        :param dict args: Named arguments (if the command requires these)
        :param bool dry_run: If ``True`` the command is not actually issued. This
                             can be used to test if the server would generate
                             errors when preparing the command (for example
                             because an argument is missing).
        :param str comment: Comment attached to the command.
        :param .VerificationConfig verification: Overrides to the default
                                                 verification handling of this
                                                 command.
        :param dict extra: Extra command options for interpretation by non-core
                           extensions (custom preprocessor, datalinks, command
                           listeners).
                           Note that Yamcs will refuse command options that it
                           does now know about. Extensions should therefore
                           register available options.
        :return: An object providing access to properties of the newly issued
                 command and updated according to command history updates.
        :rtype: .MonitoredCommand
        """
        issued_command = self._tmtc_client.issue_command(
            command, args, dry_run, comment, verification, extra, beta_args_v2
        )
        command = MonitoredCommand(issued_command._proto)

        self._command_cache[command.id] = command

        # It may be that we already received some cmdhist updates
        # before the http response returned.
        if command.id in self._cmdhist_cache:
            cmdhist = self._cmdhist_cache[command.id]
            command._process_cmdhist(cmdhist)

        return command

    def _process(self, entry):
        if entry.id in self._cmdhist_cache:
            cmdhist = self._cmdhist_cache[entry.id]
            cmdhist._update(entry.attr)
        else:
            cmdhist = CommandHistory(entry)
            self._cmdhist_cache[entry.id] = cmdhist

        command = self._command_cache.get(entry.id)
        if command:
            command._process_cmdhist(cmdhist)

        return cmdhist


class AlarmSubscription(WebSocketSubscriptionFuture):
    """
    Local object representing an alarm subscription.

    A subscription object stores the currently active
    alarms.
    """

    def __init__(self, manager):
        super(AlarmSubscription, self).__init__(manager)

        self._cache = {}
        """Value cache keyed by alarm name."""

    def get_alarm(self, name):
        """
        Returns the alarm state associated with a specific named
        alarm from local cache.

        :param str name: Fully-qualified name
        :rtype: .Alarm
        """
        return self._cache[name]

    def list_alarms(self):
        """
        Returns a snapshot of all active alarms.

        :rtype: .Alarm[]
        """
        return [self._cache[k] for k in self._cache]

    def _process(self, alarm_update):
        alarm = alarm_update.alarm
        if alarm.is_process_ok and not alarm.is_ok and alarm.is_acknowledged:
            del self._cache[alarm.name]
        else:
            self._cache[alarm.name] = alarm


class ProcessorClient:
    """Client object that groups operations linked to a specific processor."""

    def __init__(self, ctx, instance, processor):
        super(ProcessorClient, self).__init__()
        self.ctx = ctx
        self._instance = instance
        self._processor = processor

    def get_parameter_value(self, parameter, from_cache=True, timeout=10):
        """
        Retrieve the current value of the specified parameter.

        :param str parameter: Either a fully-qualified XTCE name or an alias in the
                              format ``NAMESPACE/NAME``.
        :param bool from_cache: If ``False`` this call will block until a
                                fresh value is received on the processor.
                                If ``True`` the server returns the latest
                                value instead (which may be ``None``).
        :param float timeout: The amount of seconds to wait for a fresh value.
                              (ignored if ``from_cache=True``).
        :rtype: .ParameterValue
        """
        params = {
            "fromCache": from_cache,
            "timeout": int(timeout * 1000),
        }
        parameter = adapt_name_for_rest(parameter)
        url = f"/processors/{self._instance}/{self._processor}/parameters{parameter}"
        response = self.ctx.get_proto(url, params=params)
        proto = pvalue_pb2.ParameterValue()
        proto.ParseFromString(response.content)

        # Server returns ParameterValue with only 'id' set if no
        # value existed. Convert this to ``None``.
        if proto.HasField("rawValue") or proto.HasField("engValue"):
            return ParameterValue(proto)
        return None

    def get_parameter_values(self, parameters, from_cache=True, timeout=10):
        """
        Retrieve the current value of the specified parameter.

        :param str[] parameters: List of parameter names. These may be
                                 fully-qualified XTCE name or an alias
                                 in the format ``NAMESPACE/NAME``.
        :param bool from_cache: If ``False`` this call will block until
                                fresh values are received on the processor.
                                If ``True`` the server returns the latest
                                value instead (which may be ``None``).
        :param float timeout: The amount of seconds to wait for a fresh
                              values (ignored if ``from_cache=True``).
        :return: A list that matches the length and order of the requested
                 list of parameters. Each entry contains either the
                 returned parameter value, or ``None``.
        :rtype: .ParameterValue[]
        """
        req = processing_pb2.BatchGetParameterValuesRequest()
        req.id.extend(to_named_object_ids(parameters))
        req.fromCache = from_cache
        req.timeout = int(timeout * 1000)
        url = f"/processors/{self._instance}/{self._processor}/parameters:batchGet"
        response = self.ctx.post_proto(url, data=req.SerializeToString())
        proto = processing_pb2.BatchGetParameterValuesResponse()
        proto.ParseFromString(response.content)

        pvals = []
        for parameter_id in req.id:
            match = None
            for pval in proto.value:
                if pval.id == parameter_id:
                    match = pval
                    break
            pvals.append(ParameterValue(match) if match else None)
        return pvals

    def set_parameter_value(self, parameter, value, generation_time=None):
        """
        Sets the value of the specified parameter.

        :param str parameter: Either a fully-qualified XTCE name or an alias in the
                              format ``NAMESPACE/NAME``.
        :param value: The value to set
        :param generation_time: Generation time of the values. If unset, Yamcs will
                                assign the generation time.
        :type generation_time: Optional[~datetime.datetime]
        """
        self.set_parameter_values({parameter: value}, generation_time)

    def set_parameter_values(self, values, generation_time=None):
        """
        Sets the value of multiple parameters.

        Values are specified with their native Python types. If you need
        to customize individual value generation times, use :class:`.ValueUpdate`
        instead.

        The method argument ``generation_time`` can be used to specify a custom
        generation time for all values at once. This has lower priority than
        a value-specific generation time.

        If no generation time is specified at all, Yamcs will determine one.

        :param dict values: Values keyed by parameter name. This name can be either
                            a fully-qualified XTCE name or an alias in the format
                            ``NAMESPACE/NAME``.
        :param generation_time: Generation time of the values.
        :type generation_time: Optional[~datetime.datetime]
        """
        req = processing_pb2.BatchSetParameterValuesRequest()
        for key in values:
            item = req.request.add()
            item.id.MergeFrom(to_named_object_id(key))

            value = values[key]
            value_time = generation_time
            if isinstance(values[key], ValueUpdate):
                value = values[key].value
                if values[key].generation_time:
                    value_time = values[key].generation_time

            item.value.MergeFrom(_build_value_proto(value))
            if value_time:
                item.generationTime.MergeFrom(to_server_time(value_time))
        url = f"/processors/{self._instance}/{self._processor}/parameters:batchSet"
        self.ctx.post_proto(url, data=req.SerializeToString())

    def issue_command(
        self,
        command,
        args=None,
        dry_run=False,
        comment=None,
        verification=None,
        extra=None,
        beta_args_v2=False,
    ):
        """
        Issue the given command

        :param str command: Either a fully-qualified XTCE name or an alias in the
                            format ``NAMESPACE/NAME``.
        :param dict args: named arguments (if the command requires these)
        :param bool dry_run: If ``True`` the command is not actually issued. This
                             can be used to test if the server would generate
                             errors when preparing the command (for example
                             because an argument is missing).
        :param str comment: Comment attached to the command.
        :param .VerificationConfig verification: Overrides to the default
                                                 verification handling of this
                                                 command.
        :param dict extra: Extra command options for interpretation by non-core
                           extensions (custom preprocessor, datalinks, command
                           listeners).
                           Note that Yamcs will refuse command options that it
                           does now know about. Extensions should therefore
                           register available options.
        :return: An object providing access to properties of the newly issued
                 command.
        :rtype: .IssuedCommand
        """
        req = commands_service_pb2.IssueCommandRequest()
        req.sequenceNumber = SequenceGenerator.next()
        req.dryRun = dry_run
        if comment:
            req.comment = comment
        if beta_args_v2:
            for key in beta_args_v2:
                req.args[key] = _to_argument_value(
                    beta_args_v2[key], force_string=False
                )
        elif args:
            for key in args:
                assignment = req.assignment.add()
                assignment.name = key
                assignment.value = _to_argument_value(args[key], force_string=True)

        if verification:
            if verification._disable_all:
                req.disableVerifiers = True
            else:
                for verifier in verification._disabled:
                    req.verifierConfig[verifier].disable = True
                for verifier in verification._check_windows:
                    window = verification._check_windows[verifier]
                    if window["start"]:
                        start = int(window["start"] * 1000)
                        req.verifierConfig[
                            verifier
                        ].checkWindow.timeToStartChecking = start
                    if window["stop"]:
                        stop = int(window["stop"] * 1000)
                        req.verifierConfig[
                            verifier
                        ].checkWindow.timeToStopChecking = stop

        if extra:
            for key in extra:
                req.extra[key].MergeFrom(_build_value_proto(extra[key]))

        command = adapt_name_for_rest(command)
        url = f"/processors/{self._instance}/{self._processor}/commands{command}"
        response = self.ctx.post_proto(url, data=req.SerializeToString())
        proto = commands_service_pb2.IssueCommandResponse()
        proto.ParseFromString(response.content)
        return IssuedCommand(proto)

    def list_alarms(self, start=None, stop=None):
        """
        Lists the active alarms.

        Remark that this does not query the archive. Only active alarms on the
        current processor are returned.

        :param ~datetime.datetime start: Minimum trigger time of the returned alarms
                                         (inclusive)
        :param ~datetime.datetime stop: Maximum trigger time of the returned alarms
                                        (exclusive)
        :rtype: ~collections.abc.Iterable[.Alarm]
        """
        # TODO implement continuation token on server
        params = {"order": "asc"}
        if start is not None:
            params["start"] = to_isostring(start)
        if stop is not None:
            params["stop"] = to_isostring(stop)
        # Server does not do pagination on listings of this resource.
        # Return an iterator anyway for similarity with other API methods
        url = f"/processors/{self._instance}/{self._processor}/alarms"
        response = self.ctx.get_proto(path=url, params=params)
        message = alarms_service_pb2.ListAlarmsResponse()
        message.ParseFromString(response.content)
        alarms = getattr(message, "alarms")
        return iter([_parse_alarm(alarm) for alarm in alarms])

    def set_default_calibrator(self, parameter, type, data):
        """
        Apply a calibrator while processing raw values of the specified
        parameter. If there is already a default calibrator associated
        to this parameter, that calibrator gets replaced.

        .. note::

            Contextual calibrators take precedence over the default calibrator
            See :meth:`set_calibrators` for setting contextual calibrators.

        Two types of calibrators can be applied:

        * Polynomial calibrators apply a polynomial expression of the form:
          `y = a + bx + cx^2 + ...`.

          The `data` argument must be an array of floats ``[a, b, c, ...]``.

        * Spline calibrators interpolate the raw value between a set of points
          which represent a linear curve.

          The `data` argument must be an array of ``[x, y]`` points.

        :param str parameter: Either a fully-qualified XTCE name or an alias
                              in the format ``NAMESPACE/NAME``.
        :param str type: One of ``polynomial`` or ``spline``.
        :param data: Calibration definition for the selected type.
        """
        req = mdb_override_service_pb2.UpdateParameterRequest()
        req.action = (
            mdb_override_service_pb2.UpdateParameterRequest.SET_DEFAULT_CALIBRATOR
        )
        if type:
            _add_calib(req.defaultCalibrator, type, data)

        parameter = adapt_name_for_rest(parameter)
        url = f"/mdb/{self._instance}/{self._processor}/parameters{parameter}"
        self.ctx.patch_proto(url, data=req.SerializeToString())

    def set_calibrators(self, parameter, calibrators):
        """
        Apply an ordered set of calibrators for the specified parameter.
        This replaces existing calibrators (if any).

        Each calibrator may have a context, which indicates when it its
        effects may be applied. Only the first matching calibrator is
        applied.

        A calibrator with context ``None`` is the *default* calibrator.
        There can be only one such calibrator, and is always applied at
        the end when no other contextual calibrator was applicable.

        :param str parameter: Either a fully-qualified XTCE name or an alias
                              in the format ``NAMESPACE/NAME``.
        :param .Calibrator[] calibrators: List of calibrators (either contextual or
                                          not)
        """
        req = mdb_override_service_pb2.UpdateParameterRequest()
        req.action = mdb_override_service_pb2.UpdateParameterRequest.SET_CALIBRATORS
        for c in calibrators:
            if c.context:
                context_calib = req.contextCalibrator.add()
                context_calib.context = c.context
                calib_info = context_calib.calibrator
            else:
                calib_info = req.defaultCalibrator

            _add_calib(calib_info, c.type, c.data)

        parameter = adapt_name_for_rest(parameter)
        url = f"/mdb/{self._instance}/{self._processor}/parameters{parameter}"
        self.ctx.patch_proto(url, data=req.SerializeToString())

    def clear_calibrators(self, parameter):
        """
        Removes all calibrators for the specified parameter.
        """
        self.set_default_calibrator(parameter, None, None)
        self.set_calibrators(parameter, [])

    def reset_calibrators(self, parameter):
        """
        Reset all calibrators for the specified parameter to their original MDB value.
        """
        req = mdb_override_service_pb2.UpdateParameterRequest()
        req.action = mdb_override_service_pb2.UpdateParameterRequest.RESET_CALIBRATORS

        parameter = adapt_name_for_rest(parameter)
        url = f"/mdb/{self._instance}/{self._processor}/parameters{parameter}"
        self.ctx.patch_proto(url, data=req.SerializeToString())

    def set_default_alarm_ranges(
        self,
        parameter,
        watch=None,
        warning=None,
        distress=None,
        critical=None,
        severe=None,
        min_violations=1,
    ):
        """
        Generate out-of-limit alarms for a parameter using the specified
        alarm ranges.

        This replaces any previous default alarms on this parameter.

        .. note::

            Contextual range sets take precedence over the default alarm
            ranges. See :meth:`set_alarm_range_sets` for setting contextual
            range sets.

        :param str parameter: Either a fully-qualified XTCE name or an alias
                              in the format ``NAMESPACE/NAME``.
        :param (float,float) watch: Range expressed as a tuple ``(lo, hi)``
                                    where lo and hi are assumed exclusive.
        :param (float,float) warning: Range expressed as a tuple ``(lo, hi)``
                                      where lo and hi are assumed exclusive.
        :param (float,float) distress: Range expressed as a tuple ``(lo, hi)``
                                       where lo and hi are assumed exclusive.
        :param (float,float) critical: Range expressed as a tuple ``(lo, hi)``
                                       where lo and hi are assumed exclusive.
        :param (float,float) severe: Range expressed as a tuple ``(lo, hi)``
                                     where lo and hi are assumed exclusive.
        :param int min_violations: Minimum violations before an alarm is
                                   generated.
        """
        req = mdb_override_service_pb2.UpdateParameterRequest()
        req.action = mdb_override_service_pb2.UpdateParameterRequest.SET_DEFAULT_ALARMS
        if watch or warning or distress or critical or severe:
            _add_alarms(
                req.defaultAlarm,
                watch,
                warning,
                distress,
                critical,
                severe,
                min_violations,
            )

        parameter = adapt_name_for_rest(parameter)
        url = f"/mdb/{self._instance}/{self._processor}/parameters{parameter}"
        self.ctx.patch_proto(url, data=req.SerializeToString())

    def set_alarm_range_sets(self, parameter, sets):
        """
        Apply an ordered list of alarm range sets for the specified parameter.
        This replaces existing alarm sets (if any).

        Each RangeSet may have a context, which indicates when
        its effects may be applied. Only the first matching set is
        applied.

        A RangeSet with context ``None`` represents the *default* set of
        alarm ranges.  There can be only one such set, and it is always
        applied at the end when no other set of contextual ranges is
        applicable.

        :param str parameter: Either a fully-qualified XTCE name or an alias
                              in the format ``NAMESPACE/NAME``.
        :param .RangeSet[] sets: List of range sets (either contextual or not)
        """
        req = mdb_override_service_pb2.UpdateParameterRequest()
        req.action = mdb_override_service_pb2.UpdateParameterRequest.SET_ALARMS
        for rs in sets:
            if rs.context:
                context_alarm = req.contextAlarm.add()
                context_alarm.context = rs.context
                alarm_info = context_alarm.alarm
            else:
                alarm_info = req.defaultAlarm

            _add_alarms(
                alarm_info,
                rs.watch,
                rs.warning,
                rs.distress,
                rs.critical,
                rs.severe,
                rs.min_violations,
            )

        parameter = adapt_name_for_rest(parameter)
        url = f"/mdb/{self._instance}/{self._processor}/parameters{parameter}"
        self.ctx.patch_proto(url, data=req.SerializeToString())

    def clear_alarm_ranges(self, parameter):
        """
        Removes all alarm limits for the specified parameter.
        """
        self.set_default_alarm_ranges(parameter)
        self.set_alarm_range_sets(parameter, [])

    def reset_alarm_ranges(self, parameter):
        """
        Reset all alarm limits for the specified parameter to their original MDB value.
        """
        req = mdb_override_service_pb2.UpdateParameterRequest()
        req.action = mdb_override_service_pb2.UpdateParameterRequest.RESET_ALARMS

        parameter = adapt_name_for_rest(parameter)
        url = f"/mdb/{self._instance}/{self._processor}/parameters{parameter}"
        self.ctx.patch_proto(url, data=req.SerializeToString())

    def acknowledge_alarm(self, alarm, comment=None):
        """
        Acknowledges a specific alarm.

        :param alarm: Alarm instance
        :type alarm: :class:`.Alarm`
        :param str comment: Optional comment to associate with the state
                            change.
        """
        name = adapt_name_for_rest(alarm.name)
        url = f"/processors/{self._instance}/{self._processor}"
        url += f"/alarms{name}/{alarm.sequence_number}:acknowledge"
        req = alarms_service_pb2.AcknowledgeAlarmRequest()
        if comment is not None:
            req.comment = comment
        self.ctx.post_proto(url, data=req.SerializeToString())

    def unshelve_alarm(self, alarm, comment=None):
        """
        Unshelve an alarm.

        :param alarm: Alarm instance
        :type alarm: :class:`.Alarm`
        :param str comment: Optional comment to associate with the state
                            change.
        """
        name = adapt_name_for_rest(alarm.name)
        url = f"/processors/{self._instance}/{self._processor}"
        url += f"/alarms{name}/{alarm.sequence_number}:unshelve"
        req = alarms_service_pb2.UnshelveAlarmRequest()
        self.ctx.post_proto(url, data=req.SerializeToString())

    def shelve_alarm(self, alarm, comment=None):
        """
        Shelve an alarm.

        :param alarm: Alarm instance
        :type alarm: :class:`.Alarm`
        :param str comment: Optional comment to associate with the state
                            change.
        """
        name = adapt_name_for_rest(alarm.name)
        url = f"/processors/{self._instance}/{self._processor}"
        url += f"/alarms{name}/{alarm.sequence_number}:shelve"
        req = alarms_service_pb2.ShelveAlarmRequest()
        if comment is not None:
            req.comment = comment
        self.ctx.post_proto(url, data=req.SerializeToString())

    def clear_alarm(self, alarm, comment=None):
        """
        Clear an alarm.

        .. note::
            If the reason that caused the alarm is still present, a new
            alarm instance will be generated.

        :param alarm: Alarm instance
        :type alarm: :class:`.Alarm`
        :param str comment: Optional comment to associate with the state
                            change.
        """
        name = adapt_name_for_rest(alarm.name)
        url = f"/processors/{self._instance}/{self._processor}"
        url += f"/alarms{name}/{alarm.sequence_number}:clear"
        req = alarms_service_pb2.ClearAlarmRequest()
        if comment is not None:
            req.comment = comment
        self.ctx.post_proto(url, data=req.SerializeToString())

    def create_command_connection(self, on_data=None, timeout=60):
        """
        Creates a connection for issuing multiple commands and
        following up on their acknowledgment progress.

        .. note::
            This is a convenience method that merges the functionalities
            of :meth:`create_command_history_subscription` with those of
            :meth:`issue_command`.

        :param on_data: Function that gets called with  :class:`.CommandHistory`
                        updates. Only commands issued from this connection are
                        reported.
        :param timeout: The amount of seconds to wait for the request to
                        complete.
        :type timeout: float
        :return: Future that can be used to manage the background websocket
                 subscription
        :rtype: .CommandConnection
        """
        options = commands_service_pb2.SubscribeCommandsRequest()
        options.instance = self._instance
        options.processor = self._processor
        options.ignorePastCommands = True

        manager = WebSocketSubscriptionManager(
            self.ctx, topic="commands", options=options
        )

        # Represent subscription as a future
        subscription = CommandConnection(manager, self)

        wrapped_callback = functools.partial(
            _wrap_callback_parse_cmdhist_data, subscription, on_data
        )

        manager.open(wrapped_callback)

        # Wait until a reply or exception is received
        subscription.reply(timeout=timeout)

        return subscription

    def create_command_history_subscription(self, on_data=None, timeout=60):
        """
        Create a new command history subscription.

        :param on_data: (Optional) Function that gets called with
                        :class:`.CommandHistory` updates.
        :param timeout: The amount of seconds to wait for the request to
                        complete.
        :type timeout: float
        :return: Future that can be used to manage the background websocket
                 subscription
        :rtype: .CommandHistorySubscription
        """
        options = commands_service_pb2.SubscribeCommandsRequest()
        options.instance = self._instance
        options.processor = self._processor
        options.ignorePastCommands = True

        manager = WebSocketSubscriptionManager(
            self.ctx, topic="commands", options=options
        )

        # Represent subscription as a future
        subscription = CommandHistorySubscription(manager)

        wrapped_callback = functools.partial(
            _wrap_callback_parse_cmdhist_data, subscription, on_data
        )

        manager.open(wrapped_callback)

        # Wait until a reply or exception is received
        subscription.reply(timeout=timeout)

        return subscription

    def create_packet_subscription(self, on_data, stream="tm_realtime", timeout=60):
        """
        Create a new packet subscription.

        .. versionadded:: 1.6.6

        :param on_data: Function that gets called with :class:`.Packet`
                        updates.
        :type on_data: Optional[Callable[.Packet]]
        :param stream: Stream to subscribe to.
        :type stream: str
        :param timeout: The amount of seconds to wait for the request
                        to complete.
        :type timeout: Optional[float]
        :return: A Future that can be used to manage the background websocket
                 subscription.
        :rtype: .WebSocketSubscriptionFuture
        """
        options = packets_service_pb2.SubscribePacketsRequest()
        options.instance = self._instance

        # TODO remove stream default to tm_realtime within a few releases.
        # (processor option only added as of Yamcs 5.5.x)
        # if stream:
        options.stream = stream
        # else:
        #    options.processor = self._processor

        manager = WebSocketSubscriptionManager(
            self.ctx, topic="packets", options=options
        )

        # Represent subscription as a future
        subscription = WebSocketSubscriptionFuture(manager)

        wrapped_callback = functools.partial(_wrap_callback_parse_packet_data, on_data)

        manager.open(wrapped_callback)

        # Wait until a reply or exception is received
        subscription.reply(timeout=timeout)

        return subscription

    def create_container_subscription(self, containers, on_data=None, timeout=60):
        """
        Create a new container subscription.

        .. versionadded:: 1.7.0
           Compatible with Yamcs 5.5.0 onwards

        :param containers: Container names.
        :type containers: Union[str, str[]]
        :param on_data: Function that gets called with :class:`.ContainerData`
                        updates.
        :type on_data: Optional[Callable[.ContainerData]]
        :param timeout: The amount of seconds to wait for the request
                        to complete.
        :type timeout: Optional[float]
        :return: A Future that can be used to manage the background websocket
                 subscription.
        :rtype: .ContainerSubscription
        """
        options = packets_service_pb2.SubscribeContainersRequest()
        options.instance = self._instance
        options.processor = self._processor
        if isinstance(containers, str):
            options.names.extend([containers])
        else:
            options.names.extend(containers)

        manager = WebSocketSubscriptionManager(
            self.ctx, topic="containers", options=options
        )

        # Represent subscription as a future
        subscription = ContainerSubscription(manager)

        wrapped_callback = functools.partial(
            _wrap_callback_parse_container_data, subscription, on_data
        )

        manager.open(wrapped_callback)

        # Wait until a reply or exception is received
        subscription.reply(timeout=timeout)

        return subscription

    def create_parameter_subscription(
        self,
        parameters,
        on_data=None,
        abort_on_invalid=True,
        update_on_expiration=False,
        send_from_cache=True,
        timeout=60,
    ):
        """
        Create a new parameter subscription.

        :param str[] parameters: Parameter names (or aliases).
        :param on_data: (Optional) Function that gets called with
                        :class:`.ParameterData` updates.
        :param bool abort_on_invalid: If ``True`` an error is generated when
                                      invalid parameters are specified.
        :param bool update_on_expiration: If ``True`` an update is received
                                          when a parameter value has become
                                          expired. This update holds the
                                          same value as the last known valid
                                          value, but with status set to
                                          ``EXPIRED``.
        :param bool send_from_cache: If ``True`` the last processed parameter
                                     value is sent from parameter cache.
                                     When ``False`` only newly processed
                                     parameters are received.
        :param float timeout: The amount of seconds to wait for the request
                              to complete.

        :return: A Future that can be used to manage the background websocket
                 subscription.
        :rtype: .ParameterSubscription
        """
        options = processing_pb2.SubscribeParametersRequest()
        options.instance = self._instance
        options.processor = self._processor
        options.abortOnInvalid = abort_on_invalid
        options.updateOnExpiration = update_on_expiration
        options.sendFromCache = send_from_cache
        options.id.extend(to_named_object_ids(parameters))

        manager = WebSocketSubscriptionManager(
            self.ctx, topic="parameters", options=options
        )

        # Represent subscription as a future
        subscription = ParameterSubscription(manager)

        wrapped_callback = functools.partial(
            _wrap_callback_parse_parameter_data, subscription, on_data
        )

        manager.open(wrapped_callback)

        # Wait until a reply or exception is received
        subscription.reply(timeout=timeout)

        return subscription

    def create_alarm_subscription(self, on_data=None, timeout=60):
        """
        Create a new alarm subscription.

        :param on_data: (Optional) Function that gets called with
                        :class:`.AlarmUpdate` updates.
        :param float timeout: The amount of seconds to wait for the request
                              to complete.

        :return: A Future that can be used to manage the background websocket
                 subscription.
        :rtype: .AlarmSubscription
        """
        options = alarms_service_pb2.SubscribeAlarmsRequest()
        options.instance = self._instance
        options.processor = self._processor
        manager = WebSocketSubscriptionManager(
            self.ctx, topic="alarms", options=options
        )

        # Represent subscription as a future
        subscription = AlarmSubscription(manager)

        wrapped_callback = functools.partial(
            _wrap_callback_parse_alarm_data, subscription, on_data
        )

        manager.open(wrapped_callback)

        # Wait until a reply or exception is received
        subscription.reply(timeout=timeout)

        return subscription

    def set_algorithm(self, parameter, text):
        """
        Change an algorithm text. Can only be peformed on JavaScript or Python
        algorithms.

        :param string text: new algorithm text (as it would appear in excel or XTCE)
        :param str parameter: Either a fully-qualified XTCE name or an alias
                              in the format ``NAMESPACE/NAME``.
        """
        req = mdb_override_service_pb2.UpdateAlgorithmRequest()
        req.action = mdb_override_service_pb2.UpdateAlgorithmRequest.SET
        req.algorithm.text = text

        parameter = adapt_name_for_rest(parameter)
        url = f"/mdb/{self._instance}/{self._processor}/algorithms{parameter}"
        self.ctx.patch_proto(url, data=req.SerializeToString())

    def reset_algorithm(self, parameter):
        """
        Reset the algorithm text to its original definition from MDB

        :param str parameter: Either a fully-qualified XTCE name or an alias
                              in the format ``NAMESPACE/NAME``.
        """
        req = mdb_override_service_pb2.UpdateAlgorithmRequest()
        req.action = mdb_override_service_pb2.UpdateAlgorithmRequest.RESET

        parameter = adapt_name_for_rest(parameter)
        url = f"/mdb/{self._instance}/{self._processor}/algorithms{parameter}"
        self.ctx.patch_proto(url, data=req.SerializeToString())
